from flask import Blueprint
from . import baseAPI
from . import deviceController
from . import gpioservice
from . import apiController

import json


deviceApiBlueprint = Blueprint('deviceApi', __name__)

@deviceApiBlueprint.route('/trainmote/api/v1/device/restart', methods=["POST"])
def restartDevice():
    try:
        deviceController.restartAfter(2)
        apiController.stopRunningThreads()
        gpioservice.clean()
        return "", 200
    except PermissionError as e:
        return json.dumps({"error": str(e)}), 401, baseAPI.defaultHeader()

@deviceApiBlueprint.route('/trainmote/api/v1/device/shutdown', methods=["POST"])
def shutdownDevice():
    try:
        deviceController.shutdownAfter(2)
        apiController.stopRunningThreads()
        gpioservice.clean()
        return "", 200
    except PermissionError as e:
        return json.dumps({"error": str(e)}), 401, baseAPI.defaultHeader()

@deviceApiBlueprint.route('/trainmote/api/v1/device/update', methods=["POST"])
def updateDevice():
    deviceController.update()
    apiController.stopRunningThreads()
    gpioservice.clean()
    return "", 200



from subprocess import call
import time
from threading import Thread
import os
import sys
from . import userHelper

def shutdownAfter(seconds: int):
    if userHelper.is_root() is True:
        timer = ShutDownTimer(seconds)
        timer.start()
    else:
        raise PermissionError("No root permission")

def shutdown():
    call("sudo nohup shutdown -h now", shell=True)

class ShutDownTimer(Thread):

    def __init__(self, time: int):
        Thread.__init__(self)
        self.time = time

    def run(self):
        time.sleep(self.time)
        shutdown()

def restartAfter(seconds: int):
    if userHelper.is_root() is True:
        timer = RestartTimer(seconds)
        timer.start()
    else:
        raise PermissionError("No root permission")


def restart():
    call("sudo reboot", shell=True)

class RestartTimer(Thread):
    def __init__(self, time: int):
        Thread.__init__(self)
        self.time = time

    def run(self):
        time.sleep(self.time)
        restart()

def performUpdate():
    if userHelper.is_root() is True:
        call("sudo pip3 install trainmote-module-felix-nievelstein-de -U", shell=True)
    else:
        call("pip3 install trainmote-module-felix-nievelstein-de -U", shell=True)
    os.execv(sys.argv[0], [sys.argv[0]])

def update():
    timer = UpdateTimer()
    timer.start()

class UpdateTimer(Thread):
    def run(self):
        time.sleep(2)
        performUpdate()
